from dataclasses import dataclass
from typing import List, Dict

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class DataTransaction(Transaction):
    TYPE: int = 12
    LATEST_VERSION: int = 2
    MIN_FEE: int = 100000
    data: List[Dict] = None



from waves_python.common.base58_string import Base58String


class PrivateKey(Base58String):
    LENGTH = 32

    def __init__(self, base58_encoded: str = None, byte_array: bytearray = None) -> None:
        if base58_encoded and byte_array:
            raise Exception()
        super().__init__(base58_encoded, byte_array)

    @classmethod
    def from_str(cls, base58_encoded: str):
        public_key = cls(base58_encoded=base58_encoded)
        return public_key

    @classmethod
    def from_bytes(cls, byte_array: bytearray):
        public_key = cls(byte_array=byte_array)
        return public_key

    def is_signature_valid(self, signature: bytearray):
        if len(signature) != 64:
            raise ValueError(f"Signature has wrong size in bytes. Expected: 64, actual: {len(signature)}")
        else:
            return True

    def __eq__(self, obj):
        if self == obj:
            return True
        elif obj and not isinstance(obj, self.__class__):
            return self.byte_array == obj.byte_array
        else:
            return False

    def __hash__(self):
        return hash((super().__hash__(), self.byte_array))

    def __str__(self):
        return self.encoded()



from dataclasses import dataclass

from dataclasses_json import LetterCase, dataclass_json

from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class IssueTransaction(Transaction):
    TYPE: int = 3
    LATEST_VERSION: int = 3
    min_fee: float = 100000000
    nft_min_fee: float = 100000
    name: bytearray = None
    description: bytearray = None
    quantity: float = 0
    decimals: int = 0
    is_reissuable: bool = False
    script: str = ''



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction_with_status import TransactionWithStatus


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TransactionInfo(TransactionWithStatus):
    height: int = 0



from dataclasses import dataclass, field
from typing import Any, Optional

from dataclasses_json import dataclass_json, LetterCase, config

from waves_python.account.address import Address
from waves_python.common.id import Id
from waves_python.model.lease_status import LeaseStatus
from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class LeaseInfo(NodeModel):
    lease_id: Any = field(metadata=config(field_name="id"))
    origin_transaction_id: Any
    sender: Any
    recipient: Any
    amount: float
    height: int
    status: LeaseStatus
    cancel_height: Optional[int]
    cancel_transaction_id: Any

    def __post_init__(self):
        self.lease_id = Id(base58_str=self.lease_id)
        self.sender = Address(base58_str=self.sender)
        self.recipient = Address(base58_str=self.recipient)
        self.origin_transaction_id = Id(base58_str=self.origin_transaction_id)
        self.cancel_transaction_id = Id(base58_str=self.cancel_transaction_id)



from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel

BYTE_LENGTH = 32
WAVES_STRING = "WAVES"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AssetId:
    base58_str: str


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AssetIds(NodeModel):
    ids: List[AssetId]



from dataclasses import field, dataclass
from dataclasses_json import config, dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ScriptDetails(NodeModel):
    script: str
    complexity: int = field(metadata=config(field_name="scriptComplexity"))



from dataclasses import dataclass
from typing import List, Dict

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ExchangeTransaction(Transaction):
    TYPE: int = 7
    LATEST_VERSION: int = 3
    MIN_FEE: int = 300000
    amount: float = 0
    price: float = 0
    buy_matcher_fee: float = 0
    sell_matcher_fee: float = 0
    orders: List[Dict] = None



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.application_status import ApplicationStatus
from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TransactionWithStatus(Transaction):
    application_status: ApplicationStatus = ApplicationStatus.SUCCEEDED



from waves_python.common.ByteString import ByteString


class Base64String(ByteString):

    def __init__(self, base64_encoded=None, byte_array=None) -> None:
        self.__byte_array = bytearray() if not byte_array else byte_array
        self.__encoded = base64_encoded

    @classmethod
    def empty(cls):
        public_key = cls()
        return public_key

    def byte_array(self) -> bytearray:
        return self.__byte_array

    def encoded(self) -> str:
        return self.__encoded

    def encoded_with_prefix(self) -> str:
        return f"base64:{self.__encoded}"



from dataclasses import dataclass
from typing import Any

from dataclasses_json import LetterCase, dataclass_json

from waves_python.common.asset_id import AssetId
from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class SetAssetScriptTransaction(Transaction):
    TYPE: int = 15
    LATEST_VERSION: int = 2
    min_fee: float = 100000000
    asset_id: Any = None
    script: str = ''

    def __post_init__(self):
        self.asset_id = AssetId(base58_str=self.asset_id)



from dataclasses import dataclass
from typing import Any

from dataclasses_json import dataclass_json, LetterCase

from waves_python.common.id import Id
from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class LeaseCancelTransaction(Transaction):
    TYPE: int = 9
    LATEST_VERSION: int = 3
    min_fee: float = 100000
    lease_id: Any = None

    def __post_init__(self):
        self.lease_id = Id(base58_str=self.lease_id)



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class HistoryBalance(NodeModel):
    height: int
    balance: float



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel
from waves_python.model.votes import Votes


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class BlockchainRewards(NodeModel):
    height: int
    total_waves_amount: float
    current_reward: float
    min_increment: float
    term: int
    next_check: int
    voting_interval_start: int
    voting_interval: int
    voting_threshold: int
    votes: Votes




from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction_with_status import TransactionWithStatus
from waves_python.model.block_headers import BlockHeaders


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Block(BlockHeaders):
    transactions: List[TransactionWithStatus]
    fee: float



from __future__ import annotations

from dataclasses import dataclass
from typing import List, Any, Optional, Dict

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Address(NodeModel):
    base58_str: str


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AddressList(NodeModel):
    addresses: List[str]
    height: int


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AddressBalance(NodeModel):
    address: str
    regular: int
    generating: int
    available: int
    effective: int


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AddressData(NodeModel):
    key: Optional[str]
    type: Optional[Any]
    value: Optional[Any]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AddressKeys(NodeModel):
    keys: List[str]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AddressScriptInfo(NodeModel):
    address: str
    script: str
    scriptText: str
    complexity: int
    verifierComplexity: int
    callableComplexities: Dict
    extraFee: int


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AddressScriptInfoMeta(NodeModel):
    address: str
    meta: Dict



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class BurnTransaction(Transaction):
    TYPE: int = 6
    LATEST_VERSION: int = 3
    MIN_FEE: int = 100000
    amount: float = 0




from dataclasses import dataclass, field
from typing import List, Any

from dataclasses_json import dataclass_json, LetterCase, config

from waves_python.account.address import Address
from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class BlockNxtConsensus(NodeModel):
    base_target: float = field(metadata=config(field_name="base-target"))
    generation_signature: str = field(metadata=config(field_name="generation-signature"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class BlockHeaders(NodeModel):
    version: int
    timestamp: float
    reference: str
    nxt_consensus: BlockNxtConsensus = field(metadata=config(field_name="nxt-consensus"))
    transactions_root: str
    id: str
    features: List[int]
    desired_reward: float
    generator: Any
    signature: str
    size: int = field(metadata=config(field_name="blocksize"))
    transaction_count: int
    height: int
    total_fee: float
    reward: float
    vrf: str = field(metadata=config(field_name="VRF"))

    def __post_init__(self):
        self.generator = Address(base58_str=self.generator)



from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Id(NodeModel):
    base58_str: str


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Ids(NodeModel):
    ids: List[Id]



from dataclasses import dataclass
from typing import Optional, Dict

from dataclasses_json import dataclass_json, LetterCase

from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AssetBalance(NodeModel):
    asset_id: str
    balance: float
    reissuable: bool
    min_sponsored_asset_fee: Optional[float]
    sponsor_balance: Optional[float]
    quantity: Optional[float]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AssetDistribution(NodeModel):
    has_next: bool
    last: str
    values: Dict



import base58

from waves_python.common.base58_string import Base58String


def from_string(wif: str):
    return base58.b58decode(wif)


def to_string(decoded: str):
    return base58.b58encode(decoded).decode("utf-8")


def base58_to_string(decoded: Base58String):
    return base58.b58encode(decoded.__str__()).decode("utf-8")




from dataclasses import dataclass
from typing import Any

from dataclasses_json import dataclass_json, LetterCase

from waves_python.account.address import Address
from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TransferTransaction(Transaction):
    TYPE: int = 4
    LATEST_VERSION: int = 3
    min_fee: float = 100000
    recipient: Any = None
    amount: float = 0
    attachment: str = ''

    def __post_init__(self):
        self.recipient = Address(base58_str=self.recipient)



from enum import Enum


class Profile(Enum):
    MAINNET = "https://nodes.wavesnodes.com"
    STAGENET = "https://nodes-stagenet.wavesnodes.com"
    TESTNET = "https://nodes-testnet.wavesnodes.com"
    LOCAL = "http://127.0.0.1:8081"



from typing import List, Any

import requests

from waves_python.api.profile import Profile
from waves_python.account.address import Address, AddressList, AddressBalance, AddressData, AddressKeys, AddressScriptInfo, \
    AddressScriptInfoMeta
from waves_python.common.alias import Alias
from waves_python.common.asset_id import AssetId, AssetIds
from waves_python.common.exceptions import NodeException
from waves_python.common.id import Id, Ids
from waves_python.transaction.transaction_info import TransactionInfo
from waves_python.transaction.transaction_with_status import TransactionWithStatus
from waves_python.model.asset_balance import AssetBalance, AssetDistribution
from waves_python.model.asset_details import AssetDetails
from waves_python.model.block import Block
from waves_python.model.block_headers import BlockHeaders
from waves_python.model.blockchain_rewards import BlockchainRewards
from waves_python.model.history_balance import HistoryBalance
from waves_python.model.lease_info import LeaseInfo
from waves_python.transaction.transaction import Transaction


class Node:

    def __init__(self, profile: Profile) -> None:
        self.uri = profile.value
        self.timeout = 60 * 1000
        self.headers = {'accept': 'application/json', 'content-type': 'application/json'}

    def get(self, url: str):
        response = requests.get(url=url, timeout=self.timeout, headers=self.headers)
        return response.json()

    def post(self, url: str, node_model: Any):
        response = requests.post(url=url,
                                 data=node_model.to_json(),
                                 timeout=self.timeout,
                                 headers=self.headers)
        return response.json()

    # ===============
    #    ADDRESSES
    # ===============

    def get_addresses(self) -> List[Address]:
        response_json = self.get(f'{self.uri}/addresses')
        return [Address(address_str) for address_str in response_json]

    def get_balance_for_indexes(self, idx_from: int, idx_to: int) -> List[Address]:
        response_json = self.get(f'{self.uri}/addresses/seq/{idx_from}/{idx_to}')
        return [Address(address_str) for address_str in response_json]

    def get_address_balance(self, address: Address) -> int:
        response_json = self.get(f'{self.uri}/addresses/balance/{address.base58_str}')
        return response_json.get('balance')

    def get_addresses_balances(self, addresses: List[Address], height: int) -> List[int]:
        response_json = self.post(url=f'{self.uri}/addresses/balance',
                                  node_model=AddressList(addresses=[address.base58_str for address in addresses],
                                                         height=height))
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return response_json

    def get_address_balance_details(self, address: Address) -> AddressBalance:
        response_json = self.get(f'{self.uri}/addresses/balance/details/{address.base58_str}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return AddressBalance.from_dict(response_json)

    def get_address_data(self, address: Address, matches: str = None) -> List[AddressData]:
        url = f'{self.uri}/addresses/data/{address.base58_str}'
        url += f'?matches={matches}' if matches else ''
        response_json = self.get(url)
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return [AddressData.from_dict(resp) for resp in response_json]

    def get_address_data_by_keys(self, address: Address, keys: List[str] = None) -> List[AddressData]:
        response_json = self.post(f'{self.uri}/addresses/data/{address.base58_str}', AddressKeys(keys=keys))
        if response_json.get('error') or response_json.get('error') == 0:
            return NodeException.from_dict(response_json)
        return [AddressData.from_dict(resp) for resp in response_json]

    def get_address_data_by_key(self, address: Address, key: str) -> AddressData:
        response_json = self.get(f'{self.uri}/addresses/data/{address.base58_str}/{key}')
        return AddressData.from_dict(response_json)

    def get_address_script_info(self, address: Address) -> AddressScriptInfo:
        response_json = self.get(f'{self.uri}/addresses/scriptInfo/{address.base58_str}')
        return AddressScriptInfo.from_dict(response_json)

    def get_address_script_meta(self, address: Address) -> AddressScriptInfoMeta:
        response_json = self.get(f'{self.uri}/addresses/scriptInfo/{address.base58_str}/meta')
        return AddressScriptInfoMeta.from_dict(response_json)

    # ===============
    #    ALIAS
    # ===============

    def get_aliases_by_address(self, address: Address) -> List[Alias]:
        response_json = self.get(f'{self.uri}/alias/by-address/{address.base58_str}')
        if not isinstance(response_json, list) and  response_json.get('error'):
            return NodeException.from_dict(response_json)
        return [Alias(name=resp) for resp in response_json]

    def get_address_by_alias(self, alias: Alias) -> Address:
        response_json = self.get(f'{self.uri}/alias/by-alias/{alias.name}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return Address(base58_str=response_json.get('address'))

    # ===============
    #    ASSETS
    # ===============

    def get_asset_distribution(self, asset_id: AssetId, height: int, limit: int = 100, after: str = None) -> AssetDistribution:
        url = f'{self.uri}/assets/{asset_id.base58_str}/distribution/{height}/limit/{limit}'
        url += f'?after={after}' if after else ''
        response_json = self.get(url)
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return AssetDistribution.from_dict(response_json)

    def get_assets_balance(self, address: Address) -> List[AssetBalance]:
        response_json = self.get(f'{self.uri}/assets/balance/{address.base58_str}')
        return [AssetBalance.from_dict(resp) for resp in response_json.get('balances')]

    def get_asset_balance(self, address: Address, asset_id: AssetId) -> float:
        response_json = self.get(f'{self.uri}/assets/balance/{address.base58_str}/{asset_id.base58_str}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return response_json.get('balance')

    def get_asset_details(self, asset_id: AssetId) -> AssetDetails:
        response_json = self.get(f'{self.uri}/assets/details/{asset_id.base58_str}?full=true')
        return AssetDetails.from_dict(response_json)

    def get_assets_details(self, asset_ids: AssetIds) -> List[AssetDetails]:
        response_json = self.post(f'{self.uri}/assets/details?full=true', asset_ids)
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)
        return [AssetDetails.from_dict(resp) for resp in response_json]

    def get_nft(self, address: Address, limit: int = 100, after: AssetId = None) -> List[AssetDetails]:
        url = f'{self.uri}/assets/nft/{address.base58_str}/limit/{limit}'
        url += f'?after={after.base58_str}' if after else ''
        response_json = self.get(url)
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)
        return [AssetDetails.from_dict(resp) for resp in response_json]

    # ===============
    #    BLOCKCHAIN
    # ===============

    def get_blockchain_rewards(self, height: int = None):
        url = f'{self.uri}/blockchain/rewards/'
        url += f'{height}' if height else ''
        response_json = self.get(url)
        return BlockchainRewards.from_dict(response_json)

    # ===============
    #    BLOCKS
    # ===============

    def get_blocks_height(self) -> int:
        response_json = self.get(f'{self.uri}/blocks/height')
        return response_json.get('height')

    def get_block_height_by_id(self, block_id: str) -> int:
        response_json = self.get(f'{self.uri}/blocks/height/{block_id}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return response_json.get('height')

    def get_block_height_by_timestamp(self, timestamp: int) -> int:
        response_json = self.get(f'{self.uri}/blocks/heightByTimestamp/{timestamp}')
        return response_json.get('height')

    def get_blocks_delay(self, start_block_id: str, blocks_num: int) -> int:
        response_json = self.get(f'{self.uri}/blocks/delay/{start_block_id}/{blocks_num}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return response_json.get('delay')

    def get_block_headers_by_height(self, height: int) -> BlockHeaders:
        response_json = self.get(f'{self.uri}/blocks/headers/at/{height}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return BlockHeaders.from_dict(response_json)

    def get_block_headers_by_id(self, block_id: str) -> BlockHeaders:
        response_json = self.get(f'{self.uri}/blocks/headers/{block_id}')
        return BlockHeaders.from_dict(response_json)

    def get_blocks_headers(self, height_from: int, height_to: int) -> List[BlockHeaders]:
        response_json = self.get(f'{self.uri}/blocks/headers/seq/{height_from}/{height_to}')
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)
        return [BlockHeaders.from_dict(resp) for resp in response_json]

    def get_last_block_headers(self) -> BlockHeaders:
        response_json = self.get(f'{self.uri}/blocks/headers/last')
        return BlockHeaders.from_dict(response_json)

    def get_block_by_height(self, height: int) -> Block:
        response_json = self.get(f'{self.uri}/blocks/at/{height}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return Block.from_dict(response_json)

    def get_block_by_id(self, block_id: str) -> Block:
        response_json = self.get(f'{self.uri}/blocks/{block_id}')
        if response_json.get('error'):
            return NodeException.from_dict(response_json)
        return Block.from_dict(response_json)

    def get_blocks(self, height_from: int, height_to: int) -> List[Block]:
        response_json = self.get(f'{self.uri}/blocks/seq/{height_from}/{height_to}')
        return [Block.from_dict(resp) for resp in response_json]

    def get_last_block(self) -> Block:
        response_json = self.get(f'{self.uri}/blocks/last')
        return Block.from_dict(response_json)

    def get_blocks_by_address(self, address: Address, height_from: int, height_to: int) -> List[Block]:
        response_json = self.get(f'{self.uri}/blocks/address/{address.base58_str}/{height_from}/{height_to}')
        if not isinstance(response_json, list) and  response_json.get('error'):
            return NodeException.from_dict(response_json)
        return [Block.from_dict(resp) for resp in response_json]

    # ===============
    #    NODE
    # ===============

    def get_version(self) -> str:
        response_json = self.get(f'{self.uri}/node/version')
        return response_json.get('version')

    # ===============
    #    DEBUG
    # ===============

    def get_balance_history(self, address: Address) -> List[HistoryBalance]:
        response_json = self.get(f'{self.uri}/debug/balances/history/{address.base58_str}')
        return [HistoryBalance.from_dict(resp) for resp in response_json]

    # ===============
    #    LEASING
    # ===============

    def get_active_leases(self, address: Address) -> List[LeaseInfo]:
        response_json = self.get(f'{self.uri}/leasing/active/{address.base58_str}')
        return [LeaseInfo.from_dict(resp) for resp in response_json]

    def get_lease_info_by_id(self, lease_id: Id) -> LeaseInfo:
        response_json = self.get(f'{self.uri}/leasing/info/{lease_id.base58_str}')
        return LeaseInfo.from_dict(response_json)

    def get_leases_info(self, lease_ids: Ids) -> List[LeaseInfo]:
        response_json = self.post(f'{self.uri}/leasing/info', lease_ids)
        if response_json.get('error') or response_json.get('error') == 0:
            return NodeException.from_dict(response_json)
        return [LeaseInfo.from_dict(resp) for resp in response_json]

    # ===============
    #  TRANSACTIONS
    # ===============

    def get_transaction_info(self, tx_id: Id):
        response_json = self.get(f'{self.uri}/transactions/info/{tx_id.base58_str}')
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)
        return TransactionInfo.from_dict(response_json)

    def get_transactions_by_address(self, address: Address, limit: int = 100, after_id: Id = None) -> List[Transaction]:
        url = f'{self.uri}/transactions/address/{address.base58_str}/limit/{limit}'
        url += f'?after={after_id.base58_str}' if after_id else ''
        response_json = self.get(url)
        if isinstance(response_json, list):
            response_json = response_json[0]
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)

        return [TransactionInfo.from_dict(resp) for resp in response_json]

    def get_transaction_status(self, tx_id: Id) -> TransactionWithStatus:
        response_json = self.get(f'{self.uri}/transactions/status?id={tx_id.base58_str}')
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)
        return TransactionWithStatus.from_dict(response_json[0])

    def get_transactions_status(self, tx_ids: Ids):
        response_json = self.post(f'{self.uri}/transactions/status', tx_ids)
        return [TransactionWithStatus.from_dict(resp) for resp in response_json]

    def get_unconfirmed_transaction(self, tx_id: Id):
        response_json = self.get(f'{self.uri}/transactions/unconfirmed/info/{tx_id.base58_str}')
        if not isinstance(response_json, list) and response_json.get('error'):
            return NodeException.from_dict(response_json)
        return TransactionInfo.from_dict(response_json)

    def get_unconfirmed_transactions(self):
        response_json = self.get(f'{self.uri}/transactions/unconfirmed')
        return [TransactionInfo.from_dict(resp) for resp in response_json]

    def get_unconfirmed_transaction_size(self) -> int:
        response_json = self.get(f'{self.uri}/transactions/unconfirmed/size')
        return response_json.get('size')

    # ===============
    #     UTILS
    # ===============

    def generate_random_seed(self) -> str:
        response_json = self.get(f'{self.uri}/utils/seed')
        return response_json.get('seed')



from setuptools import setup

setup(
    name='waves_python',
    version='1.0.1',
    packages=['waves_python', 'waves_python.api', 'waves_python.util', 'waves_python.model',
              'waves_python.common', 'waves_python.account', 'waves_python.transaction'],
    url='https://github.com/wavesplatform/waves-python',
    license='',
    install_requires=['requests',
                      'base58',
                      'dataclasses-json'
                      ],
    author='Dmitry Rozhaev',
    author_email='admin@waveslabs.com',
    description='Python library for interacting with the Waves blockchain.'
)



from dataclasses import dataclass
from typing import Any, Optional

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction_or_order import TransactionOrOrder


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Transaction(TransactionOrOrder):
    version: Optional[int] = None
    fee: Optional[float] = None
    timestamp: Optional[float] = None
    sender: Optional[Any] = None
    chain_id: Optional[int] = None
    type: Optional[int] = None




class ByteString:

    def byte_array(self) -> bytearray:
        raise NotImplementedError()

    def encoded(self) -> str:
        raise NotImplementedError()

    def encoded_with_prefix(self) -> str:
        raise NotImplementedError()



from dataclasses import dataclass
from typing import Optional, Any

from dataclasses_json import dataclass_json, LetterCase

from waves_python.account.address import Address
from waves_python.common.asset_id import AssetId
from waves_python.common.id import Id
from waves_python.model.node_model import NodeModel
from waves_python.model.script_details import ScriptDetails


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AssetDetails(NodeModel):
    asset_id: Any
    issue_height: int
    issue_timestamp: float
    issuer: Any
    issuer_public_key: str
    name: str
    description: str
    decimals: int
    reissuable: bool
    quantity: float
    scripted: bool
    min_sponsored_asset_fee: Optional[float]
    origin_transaction_id: Any
    script_details: Optional[ScriptDetails] = None

    def __post_init__(self):
        self.asset_id = AssetId(base58_str=self.asset_id)
        self.issuer = Address(base58_str=self.issuer)
        self.origin_transaction_id = Id(base58_str=self.origin_transaction_id)



from dataclasses import dataclass
from typing import Any, Optional

from dataclasses_json import dataclass_json, LetterCase

from waves_python.common.id import Id
from waves_python.model.node_model import NodeModel


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TransactionOrOrder(NodeModel):
    id: Optional[Any] = None
    public_key: Optional[str] = None
    chain_id: Optional[Any] = None

    def __post_init__(self):
        self.id = Id(base58_str=self.id)



# from waves_python.transaction.transaction import Transaction
#
#
# class InvokeScriptTransaction(Transaction):
#     pass
from dataclasses import dataclass
from typing import Any, List

from dataclasses_json import dataclass_json, LetterCase

from waves_python.account.address import Address
from waves_python.common.asset_id import AssetId
from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Function:
    name: str
    args: List[str]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Amount:
    value: float
    asset_id: Any

    def __post_init__(self):
        self.asset_id = AssetId(base58_str=self.asset_id)


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class InvokeScriptTransaction(Transaction):
    TYPE: int = 16
    LATEST_VERSION: int = 2
    d_app: Any = None
    min_fee: float = 500000
    function: Function = None
    payments: List[Amount] = None



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class NodeModel:

    def __str__(self):
        return f"{self.__class__.__name__}{self.to_json()}"





from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from waves_python.transaction.transaction import Transaction


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class SetScriptTransaction(Transaction):
    TYPE: int = 13
    LATEST_VERSION: int = 2
    min_fee: float = 1000000
    script: str = ''
